from unittest import result
import mugrade
import numpy as np

def objects_equal(value,ref):
    """ Test if two objects are equal according to our autograding rules.
    We have to do this manually so that numpy arrays are compard properly. """

    try:
        if isinstance(ref, dict):
            if sorted(value.keys()) != sorted(ref.keys()):
                return False
            for k,v in ref.items():
                if not objects_equal(v, value[k]):
                    return False
            return True
    
        elif isinstance(ref, list) or isinstance(ref, tuple):
            if len(value) != len(ref):
                return False
            for a,b in zip(ref, value):
                if not objects_equal(a, b):
                    return False
            return True
    
        elif isinstance(ref, np.ndarray):
            if value.shape != ref.shape:
                return False
            if not np.allclose(ref, value, atol=1e-8):
                return False
            return True
          
        elif isinstance(ref, float) or isinstance(ref, np.float32):
          return np.allclose(value, ref)
    
        else:
            return value == ref
    except:
        return False

def test_parse_stackoverflow_data():
    df = parse_stackoverflow_data()
    with mugrade.test:
        assert df.shape == (65679, 23)
    with mugrade.test:
        assert sorted(df.columns) == ['Age', 'CareerSat', 'CodeRevHrs', 'ConvertedComp', 'Dependents', 'DevEnvironVSC', 'DevTypeFullStack', 'EdLevel', 'EduOtherMOOC', 'Extraversion', 'GenderIsMan', 'Hobbyist', 'MgrIdiot', 'MgrWant', 'OpSys', 'OpenSourcer', 'OrgSize', 'Student', 'UndergradMajorIsComputerScience', 'UnitTestsProcess', 'WorkWeekHrs', 'YearsCode', 'YearsCodePro']
    with mugrade.test:
        assert list(map(str,df.dtypes)) == ['float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64']
    with mugrade.test:
        assert objects_equal(df["WorkWeekHrs"].mean(), 37.29011654417699)

def submit_parse_stackoverflow_data():
    df = parse_stackoverflow_data()
    mugrade.submit(df["CareerSat"].values[:10])
    mugrade.submit(df["CareerSat"].mean())
    mugrade.submit(df["OrgSize"].mean())

def test_split_data():
    result = split_data(parse_stackoverflow_data())
    with mugrade.test:
        assert result[0].shape == (52544, 23)
    with mugrade.test:
        assert result[1].shape == (52544,)
    with mugrade.test:
        assert objects_equal(result[0][:,0].mean(), 0.24120736906211937)

def submit_split_data():
    result = split_data(parse_stackoverflow_data())
    mugrade.submit(result[2].shape)
    mugrade.submit(result[3].shape)
    mugrade.submit(result[2][:,3].mean())

def test_LinearRegression():
    X_train, y_train, X_val, y_val = split_data(parse_stackoverflow_data())
    lin_reg = LinearRegression(X_train, y_train)
    with mugrade.test:
        assert objects_equal(squared_error(lin_reg.predict(X_train), y_train), 1.3088825148602046)
    with mugrade.test:
        assert objects_equal(lin_reg.predict(X_val)[:10], [1.0954934034668686, 1.3511238062974786, 0.5532071548706642, 1.222519611017453, 1.0474787739240217, 0.8142891181374975, 1.1403305828470338, 0.84951709680783, 1.2239501989396095, 1.2250600183818339])

def submit_LinearRegression():
    X_train, y_train, X_val, y_val = split_data(parse_stackoverflow_data())
    lin_reg = LinearRegression(X_train, y_train)
    mugrade.submit(squared_error(lin_reg.predict(X_val), y_val))
    mugrade.submit(lin_reg.predict(X_train)[:10])

def test_evaluate_linear_regression():
    X_train, y_train, X_val, y_val = split_data(parse_stackoverflow_data())
    with mugrade.test:
        assert objects_equal(evaluate_linear_regression(X_train, y_train, X_train, y_train), (1.3088825148602046, 1.3730471093462553))

def submit_evaluate_linear_regression():
    X_train, y_train, X_val, y_val = split_data(parse_stackoverflow_data())
    mugrade.submit(evaluate_linear_regression(X_train, y_train, X_val, y_val))
